"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ajv_1 = __importDefault(require("ajv"));
var ajv_keywords_1 = __importDefault(require("ajv-keywords"));
var json_schema_draft_06_json_1 = __importDefault(require("ajv/lib/refs/json-schema-draft-06.json"));
var cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
var defaults_1 = __importDefault(require("lodash/defaults"));
var every_1 = __importDefault(require("lodash/every"));
var findIndex_1 = __importDefault(require("lodash/findIndex"));
var findKey_1 = __importDefault(require("lodash/findKey"));
var isArray_1 = __importDefault(require("lodash/isArray"));
var isBoolean_1 = __importDefault(require("lodash/isBoolean"));
var map_1 = __importDefault(require("lodash/map"));
var pickBy_1 = __importDefault(require("lodash/pickBy"));
var reduce_1 = __importDefault(require("lodash/reduce"));
var startsWith_1 = __importDefault(require("lodash/startsWith"));
var trimStart_1 = __importDefault(require("lodash/trimStart"));
var utils = __importStar(require("../../utils"));
var DataTypes_1 = require("../DataTypes");
var ajv = new ajv_1.default();
ajv_keywords_1.default(ajv, ['regexp', 'formatMaximum', 'formatMinimum']);
ajv.addMetaSchema(json_schema_draft_06_json_1.default);
exports.FULL_TEXT_SLUG = 'full_text_search';
var DEFAULT_DELIMITER = '___';
exports.filter = function (filters, collection) {
    ajv.removeSchema(/^.*$/);
    var validators = isArray_1.default(filters)
        ? filters.map(function (s) { return ajv.compile(s); })
        : [ajv.compile(filters)];
    if (isArray_1.default(collection)) {
        return collection.filter(function (m) { return every_1.default(validators, function (v) { return v(m); }); });
    }
    return pickBy_1.default(collection, function (m) { return every_1.default(validators, function (v) { return v(m); }); });
};
exports.createFullTextSearchFilter = function (schema, term) {
    var stringKeys = reduce_1.default(schema.properties, function (carry, item, key) {
        if (item.type === 'string') {
            carry.push(key);
        }
        return carry;
    }, []);
    var filter = {
        $id: utils.randomString(),
        title: exports.FULL_TEXT_SLUG,
        anyOf: [
            {
                description: "Any field contains " + term,
                anyOf: stringKeys.map(function (key) {
                    var _a;
                    return ({
                        properties: (_a = {},
                            _a[key] = {
                                type: 'string',
                                regexp: {
                                    pattern: utils.regexEscape(term),
                                    flags: 'i',
                                },
                            },
                            _a),
                        required: [key],
                    });
                }),
            },
        ],
    };
    return filter;
};
exports.upsertFullTextSearch = function (schema, filters, term) {
    if (filters === void 0) { filters = []; }
    var searchFilter = exports.createFullTextSearchFilter(schema, term);
    var existingSearchIndex = findIndex_1.default(filters, { title: exports.FULL_TEXT_SLUG });
    if (existingSearchIndex > -1) {
        filters.splice(existingSearchIndex, 1, searchFilter);
    }
    else {
        filters.push(searchFilter);
    }
    return filters;
};
exports.removeFullTextSearch = function (filters) {
    return filters.filter(function (f) { return f.title !== exports.FULL_TEXT_SLUG; });
};
exports.createFilter = function (schema, signatures) {
    var anyOf = signatures.map(function (_a) {
        var field = _a.field, operator = _a.operator, value = _a.value;
        var subSchema = schema.properties[field];
        var model = DataTypes_1.getDataModel(subSchema);
        if (!model || !model.operators.hasOwnProperty(operator)) {
            return {};
        }
        return model.createFilter(field, operator, value, subSchema);
    });
    return {
        $id: utils.randomString(),
        anyOf: anyOf,
    };
};
exports.decodeFilter = function (schema, filter) {
    var signatures = filter.anyOf.map(function (f) {
        if (!f) {
            return null;
        }
        if (!f.properties &&
            (!f.anyOf || !f.anyOf.length || !f.anyOf[0].properties)) {
            return null;
        }
        var schemaField = f.anyOf
            ? Object.keys(f.anyOf[0].properties).shift()
            : Object.keys(f.properties).shift();
        var subSchema = schema.properties[schemaField];
        var model = DataTypes_1.getDataModel(subSchema);
        if (!model) {
            return {};
        }
        return model.decodeFilter(f);
    });
    return signatures.filter(function (s) { return s !== null; });
};
exports.getOperators = function (schema, field) {
    var subSchema = schema.properties[field];
    var model = DataTypes_1.getDataModel(subSchema);
    if (!model) {
        return [];
    }
    return map_1.default(model.operators, function (value, key) { return ({
        slug: key,
        label: value.getLabel(subSchema),
    }); });
};
var flattenAccumulator = function (schema, delimiter, parentKey, accumulator) {
    if (parentKey === void 0) { parentKey = ''; }
    if (!accumulator) {
        accumulator = cloneDeep_1.default(schema);
        if (schema.properties) {
            accumulator.properties = {};
        }
    }
    if (schema.anyOf && isArray_1.default(schema.anyOf)) {
        accumulator.anyOf = schema.anyOf.map(function (subSchema) {
            return flattenAccumulator(subSchema, delimiter, parentKey);
        });
    }
    return reduce_1.default(schema.properties, function (result, value, key) {
        if (isBoolean_1.default(value)) {
            return accumulator;
        }
        var newKey = parentKey + delimiter + key;
        if (value.type !== 'object' ||
            (findKey_1.default(value.properties, { description: 'key' }) &&
                findKey_1.default(value.properties, { description: 'value' }))) {
            if (parentKey) {
                result.properties[newKey] = defaults_1.default(value, { title: key });
                return result;
            }
            result.properties[key] = value;
            return result;
        }
        return flattenAccumulator(value, delimiter, newKey, result);
    }, accumulator);
};
exports.flattenSchema = function (schema, delimiter) {
    if (delimiter === void 0) { delimiter = DEFAULT_DELIMITER; }
    return flattenAccumulator(schema, delimiter);
};
exports.unflattenSchema = function (schema, delimiter) {
    if (delimiter === void 0) { delimiter = DEFAULT_DELIMITER; }
    var base = cloneDeep_1.default(schema);
    if (schema.properties) {
        base.properties = {};
    }
    var unflattenedSchema = reduce_1.default(schema.properties, function (result, value, field) {
        if (!startsWith_1.default(field, delimiter)) {
            result.properties[field] = value;
            return result;
        }
        var keys = trimStart_1.default(field, delimiter).split(delimiter);
        var lastKey = keys.pop();
        var head = result;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            if (!head.properties[key]) {
                head.properties[key] = {
                    type: 'object',
                    properties: {},
                };
            }
            head = head.properties[key];
        }
        head.properties[lastKey] = value;
        return result;
    }, base);
    if (unflattenedSchema.anyOf) {
        unflattenedSchema.anyOf = unflattenedSchema.anyOf.map(function (subSchema) {
            return exports.unflattenSchema(subSchema, delimiter);
        });
    }
    if (unflattenedSchema.required) {
        var requiredFields = unflattenedSchema.required;
        unflattenedSchema.required = [];
        requiredFields.forEach(function (requiredField) {
            var fields = trimStart_1.default(requiredField, delimiter).split(delimiter);
            var head = unflattenedSchema;
            for (var _i = 0, fields_1 = fields; _i < fields_1.length; _i++) {
                var field = fields_1[_i];
                if (!head.required) {
                    head.required = [];
                }
                head.required.push(field);
                head = head.properties[field];
            }
        });
    }
    return unflattenedSchema;
};
