"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = __importDefault(require("color"));
var cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
var find_1 = __importDefault(require("lodash/find"));
var forEach_1 = __importDefault(require("lodash/forEach"));
var get_1 = __importDefault(require("lodash/get"));
var isObject_1 = __importDefault(require("lodash/isObject"));
var matchOperatorsRe = /[|\\{}()[\]^$+*?.]/g;
exports.isLight = function (color) {
    return color_1.default(color).luminosity() > 0.5;
};
exports.opaque = function (color, opacity) {
    return new color_1.default(color_1.default(color).array().concat([opacity])).string();
};
exports.lighten = function (color) {
    return color_1.default(color)
        .fade(0.3)
        .string();
};
exports.darken = function (color) {
    return color_1.default(color)
        .darken(0.2)
        .string();
};
exports.fade = function (color) {
    return color_1.default(color)
        .fade(0.95)
        .string();
};
exports.blacken = function (color) {
    return color_1.default(color)
        .darken(0.6)
        .string();
};
exports.normal = function (props) {
    return get_1.default(props.theme, 'weights.0');
};
exports.bold = function (props) {
    return get_1.default(props.theme, 'weights.1');
};
exports.px = function (n) { return (typeof n === 'number' ? n + 'px' : n); };
exports.randomString = function (length) {
    if (length === void 0) { length = 16; }
    var text = '';
    var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (var i = 0; i < length; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
};
exports.stopPropagation = function (e) {
    e.stopPropagation();
};
exports.stopEvent = function (e) {
    e.preventDefault();
    e.stopPropagation();
};
exports.getColorFromTheme = function (theme) { return function (colorString) {
    return get_1.default(theme, "colors." + colorString);
}; };
exports.getColoringType = function (props) {
    var type = find_1.default(Object.keys(props), function (b) {
        return !!props[b] && !!find_1.default(Object.keys(props.theme.colors), function (k) { return k === b; });
    });
    return type;
};
exports.getColor = function (props, key, shade) {
    if (get_1.default(props, key)) {
        return shadeCustomColor(get_1.default(props, key), shade);
    }
    var type = exports.getColoringType(props);
    if (type) {
        var color = exports.getColorFromTheme(props.theme)(type);
        if (isObject_1.default(color)) {
            return color[shade];
        }
        else {
            return shadeCustomColor(color, shade);
        }
    }
};
var shadeCustomColor = function (color, shade) {
    if (shade === 'main') {
        return color;
    }
    if (shade === 'light') {
        return exports.lighten(color);
    }
    return exports.darken(color);
};
exports.monospace = function (props) {
    return props.monospace ? "font-family: " + props.theme.monospace : null;
};
exports.emphasized = function (props) {
    return "height: " + exports.px(props.emphasized ? props.theme.space[5] : props.theme.space[4]);
};
exports.regexEscape = function (str) {
    return str.replace(matchOperatorsRe, '\\$&');
};
exports.stripSchemaFormats = function (schema, whitelist) {
    if (whitelist === void 0) { whitelist = []; }
    var newSchema = cloneDeep_1.default(schema);
    var _strip = function (schema) {
        if (schema.format && whitelist.indexOf(schema.format) === -1) {
            delete schema.format;
        }
        if (schema.properties) {
            forEach_1.default(schema.properties, function (subSchema) {
                _strip(subSchema);
            });
        }
        if (schema.items) {
            _strip(schema.items);
        }
    };
    _strip(newSchema);
    return newSchema;
};
exports.disallowAdditionalProperties = function (schema) {
    var newSchema = cloneDeep_1.default(schema);
    var disallow = function (schema) {
        if (schema.additionalProperties) {
            schema.additionalProperties = false;
        }
        if (schema.properties) {
            forEach_1.default(schema.properties, function (subSchema) {
                disallow(subSchema);
            });
        }
    };
    disallow(newSchema);
    return newSchema;
};
exports.getAngleBetweenPoints = function (p1, p2) {
    return (Math.atan2(p2.y - p1.y, p2.x - p1.x) * 180) / Math.PI;
};
exports.constrainNumber = function (value, lower, upper) {
    if (lower === void 0) { lower = 0; }
    if (upper === void 0) { upper = 100; }
    return Math.min(Math.max(value, lower), upper);
};
