import Color from 'color';
import cloneDeep from 'lodash/cloneDeep';
import find from 'lodash/find';
import forEach from 'lodash/forEach';
import get from 'lodash/get';
import isObject from 'lodash/isObject';
var matchOperatorsRe = /[|\\{}()[\]^$+*?.]/g;
export var isLight = function (color) {
    return Color(color).luminosity() > 0.5;
};
export var opaque = function (color, opacity) {
    return new Color(Color(color).array().concat([opacity])).string();
};
export var lighten = function (color) {
    return Color(color)
        .fade(0.3)
        .string();
};
export var darken = function (color) {
    return Color(color)
        .darken(0.2)
        .string();
};
export var fade = function (color) {
    return Color(color)
        .fade(0.95)
        .string();
};
export var blacken = function (color) {
    return Color(color)
        .darken(0.6)
        .string();
};
export var normal = function (props) {
    return get(props.theme, 'weights.0');
};
export var bold = function (props) {
    return get(props.theme, 'weights.1');
};
export var px = function (n) { return (typeof n === 'number' ? n + 'px' : n); };
export var randomString = function (length) {
    if (length === void 0) { length = 16; }
    var text = '';
    var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (var i = 0; i < length; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
};
export var stopPropagation = function (e) {
    e.stopPropagation();
};
export var stopEvent = function (e) {
    e.preventDefault();
    e.stopPropagation();
};
export var getColorFromTheme = function (theme) { return function (colorString) {
    return get(theme, "colors." + colorString);
}; };
export var getColoringType = function (props) {
    var type = find(Object.keys(props), function (b) {
        return !!props[b] && !!find(Object.keys(props.theme.colors), function (k) { return k === b; });
    });
    return type;
};
export var getColor = function (props, key, shade) {
    if (get(props, key)) {
        return shadeCustomColor(get(props, key), shade);
    }
    var type = getColoringType(props);
    if (type) {
        var color = getColorFromTheme(props.theme)(type);
        if (isObject(color)) {
            return color[shade];
        }
        else {
            return shadeCustomColor(color, shade);
        }
    }
};
var shadeCustomColor = function (color, shade) {
    if (shade === 'main') {
        return color;
    }
    if (shade === 'light') {
        return lighten(color);
    }
    return darken(color);
};
export var monospace = function (props) {
    return props.monospace ? "font-family: " + props.theme.monospace : null;
};
export var emphasized = function (props) {
    return "height: " + px(props.emphasized ? props.theme.space[5] : props.theme.space[4]);
};
export var regexEscape = function (str) {
    return str.replace(matchOperatorsRe, '\\$&');
};
export var stripSchemaFormats = function (schema, whitelist) {
    if (whitelist === void 0) { whitelist = []; }
    var newSchema = cloneDeep(schema);
    var _strip = function (schema) {
        if (schema.format && whitelist.indexOf(schema.format) === -1) {
            delete schema.format;
        }
        if (schema.properties) {
            forEach(schema.properties, function (subSchema) {
                _strip(subSchema);
            });
        }
        if (schema.items) {
            _strip(schema.items);
        }
    };
    _strip(newSchema);
    return newSchema;
};
export var disallowAdditionalProperties = function (schema) {
    var newSchema = cloneDeep(schema);
    var disallow = function (schema) {
        if (schema.additionalProperties) {
            schema.additionalProperties = false;
        }
        if (schema.properties) {
            forEach(schema.properties, function (subSchema) {
                disallow(subSchema);
            });
        }
    };
    disallow(newSchema);
    return newSchema;
};
export var getAngleBetweenPoints = function (p1, p2) {
    return (Math.atan2(p2.y - p1.y, p2.x - p1.x) * 180) / Math.PI;
};
export var constrainNumber = function (value, lower, upper) {
    if (lower === void 0) { lower = 0; }
    if (upper === void 0) { upper = 100; }
    return Math.min(Math.max(value, lower), upper);
};
